﻿/**
 *  \file alakzat3_main.cpp
 *  Alakzatlab2 deladat továbbfejlesztése:
 *  Bővítjük poligonnal, majd sablont is használunk.
 *
 *  ELKESZULT makró értékei:
 *  == 1 - Van Poligon osztály, amit önmagában kipróbálunk, copy és assign még nincs használva.
 *  >= 2 - Poligon assign is tesztelve van.
 *  >= 3 - Poligon assing és copy is tesztelve van
 *  == 4 - Átemelésre került az előző heti Rajztabla osztály a projektbe, poligon nélküli teszt
 *  >= 5 - Rajztabla osztállyal együtt mindent kipróbálunk
 *  >= 6 - Szakasz klónozható
 *  >= 7 - Rajztabla is másolható, de csak szakaszt teszünk bele.
 *  >= 8 - Szorgalmi: Rajztabla is másolható. Poligon is klónozható.
 *  >= 9 - Rajztabla fixArray sablont használ
 */
#define ELKESZULT 1

#include <iostream>
#include <fstream>
#include "gtest_lite.h"
#include "szakasz.h"
#include "kor.h"

#if !defined(ELKESZULT) || ELKESZULT >= 1
# include "poligon.h"
#endif // ELKESZULT

#if ELKESZULT >= 3
#include "rajztabla.h"
#endif // ELKESZULT

using std::cout;
using std::endl;

int main(int argc, char** argv) {
    GTINIT(std::cin);     // Csak C(J)PORTA működéséhez kell

#if ELKESZULT == 1
    TEST(Poligon, size) {
        Poligon poli1(Pont(120,140), GREEN);
        EXPECT_EQ(1u, poli1.size()) << "*** Nem jo a kezdo merete ***" << endl;
        poli1.add(Pont(120,200));
        cout << "add pont: " << poli1 << endl;
        poli1.add(Pont(250,200));
        cout << "add pont: " << poli1 << endl;
        EXPECT_EQ(3u, poli1.size()) << "*** Nem jo a poligon merete ***" << endl;
        /// Megnézzük a pontokat, hogy jók-e a koordináták
        EXPECT_EQ(Pont(120,140), poli1[0]);
        EXPECT_EQ(Pont(120,200), poli1[1]);
        EXPECT_EQ(Pont(250,200), poli1[2]);
    } ENDM
#endif // ELKESZULT

#if ELKESZULT >= 2 && ELKESZULT != 6 && ELKESZULT != 7 && ELKESZULT != 8
    TEST(Poligon, assign) {
        Poligon* poli1 = new Poligon(Pont(120,140), GREEN);
        poli1->add(Pont(120,200));
        poli1->add(Pont(250,200));
        cout << "poli1: " << *poli1 << endl;
        Poligon poli2(Pont(), BLACK);

        /// Készitünk egy másolatot értékadással:
        poli2 = *poli1;
        cout << "poli2: " << poli2 << endl;
        EXPECT_EQ(GREEN, poli2.getsz()) << "*** Nem kapott erteket az alaposztaly!\n";
        EXPECT_EQ(3u, poli2.size()) << "*** Nem jo a poligon merete ***" << endl;
        /// Megnézzük a pontokat, hogy jók-e a koordináták
        EXPECT_EQ(Pont(120,140), poli2[0]);
        EXPECT_EQ(Pont(120,200), poli2[1]);
        EXPECT_EQ(Pont(250,200), poli2[2]);
        cout << "Most toroljk az eredeti poligont\n";
        delete poli1;
        EXPECT_EQ(Pont(120,200), poli2[1]) << "*** Memoriaszemet? Biztos, hogy jol mukodik az assign? ***\n";
    } ENDM
#endif // ELKESZULT

#if ELKESZULT >= 3 && ELKESZULT != 6 && ELKESZULT != 7
    TEST(Poligon, copy) {
        Poligon* poli3 = new Poligon(Pont(120,140), GREEN);
        poli3->add(Pont(120,200));
        poli3->add(Pont(250,200));
        cout << "poli3: " << *poli3 << endl;

        /// Készitünk egy másolatot  copy kontruktorral.
        Poligon poli4 = *poli3;
        cout << "poli4: " << poli4 << endl;
        EXPECT_EQ(GREEN, poli4.getsz()) << "*** Rosszul inicializálódott alaposztaly!\n";
        EXPECT_EQ(Pont(120,140), poli4[0]);
        EXPECT_EQ(Pont(120,200), poli4[1]);
        EXPECT_EQ(Pont(250,200), poli4[2]);
        /// Töröljük az eredeti poligont
        delete poli3;
        EXPECT_EQ(Pont(120,200), poli4[1]) << "*** Memoriaszemet? Biztos, hogy jol mukodik a copy ctor? ***\n";
    } ENDM
#endif // ELKESZULT


#if ELKESZULT == 4
    TEST(Rajztabla, Szakasz.es.kor) {
        Rajztabla tabla;
        cout << "tabla.size: " << tabla.size() << endl;
        EXPECT_EQ(0u, tabla.size()) << "*** Nem jo a kezdo meret ***" << endl;
        tabla.felrak(new Szakasz(Pont(20,40), 100, 0, WHITE));
        tabla.felrak(new Kor(Pont(120,140), 50, BLUE));
        cout << "tabla.size: " << tabla.size() << endl;
        EXPECT_EQ(2u, tabla.size()) << "*** Nem jo a meret ***" << endl;
        cout << "\nRajztabla rajzol:\n";
        tabla.rajzol();
        Pont delta(100, -10);
        cout << "\nRajztabla mozgat delta=" << delta << ":" << endl;
        tabla.mozgat(delta);

        cout << "\nRajztabla torol:\n";
        tabla.torol();
        EXPECT_EQ(0u, tabla.size()) << "*** Torles utan nem jo meret ***" << endl;
        cout << "tabla.size: " << tabla.size() << endl;

        tabla.felrak(new Kor(Pont(10,20), 50, RED));
        cout << "\nRajztabla rajzol:\n";
        tabla.rajzol();
    } ENDM
#endif

#if ELKESZULT == 5
    TEST(Rajztabla, Poligonal) {
        Rajztabla tabla;
        tabla.felrak(new Szakasz(Pont(20,40), 100, 0, WHITE));
        tabla.felrak(new Kor(Pont(120,140), 50, BLUE));
        Poligon* poli = new Poligon(Pont(120,140), GREEN);
        poli->add(Pont(120,200));
        poli->add(Pont(250,200));
        tabla.felrak(poli);
        tabla.rajzol();
    } ENDM
#endif // ELKESZULT

#if ELKESZULT == 6
    TEST(Szakasz, clone) {
        Szakasz* psz1 = new Szakasz(Pont(20,40), 100, 0, WHITE);
        cout << *psz1 << endl;
        Szakasz* psz2 = psz1->clone();
        cout << *psz2 << endl;
        EXPECT_EQ(psz1->getsz(), psz2->getsz());
        EXPECT_EQ(psz1->getp0(), psz2->getp0());
        EXPECT_EQ(psz1->getpv(), psz2->getpv());
        delete psz1;
        EXPECT_EQ(Pont(120,40), psz2->getpv());
        delete psz2;
    } ENDM
#endif // ELKESZULT


#if ELKESZULT == 7
    TEST(Rajztabla, assign) {
        Rajztabla tabla;
        tabla.felrak(new Szakasz(Pont(20,40), 100, 0, WHITE));
        cout << "tabla rajzol::\n";
        tabla.rajzol();
        EXPECT_EQ(1u, tabla.size());
        { // uj blokk, ebben jön létre az új tabla
            Rajztabla ujtabla;
            ujtabla = tabla;
            ujtabla.felrak(new Szakasz(Pont(40,40), 100, 0, WHITE));
            cout << "\nujtabla rajzol:\n";
            ujtabla.rajzol();
            EXPECT_EQ(2u, ujtabla.size());
        }
        cout << "\ntabla rajzol:\n";
        tabla.rajzol();
        EXPECT_EQ(1u, tabla.size());
    } ENDM
#endif // ELKESZULT

#if ELKESZULT == 8
    TEST(Rajztabla, assign.full) {
        Rajztabla tabla;
        tabla.felrak(new Szakasz(Pont(20,40), 100, 0, WHITE));
        Poligon* poli = new Poligon(Pont(120,140), GREEN);
        poli->add(Pont(120,200));
        poli->add(Pont(250,200));
        tabla.felrak(poli);
        tabla.rajzol();
        cout << "tabla rajzol::\n";
        tabla.rajzol();
        EXPECT_EQ(2u, tabla.size());
        { // uj blokk, ebben jön létre az új tabla
            Rajztabla ujtabla;
            ujtabla = tabla;
            ujtabla.felrak(new Szakasz(Pont(40,40), 100, 0, WHITE));
            cout << "\nujtabla rajzol:\n";
            ujtabla.rajzol();
            EXPECT_EQ(3u, ujtabla.size());
        }
        cout << "\ntabla rajzol:\n";
        tabla.rajzol();
        EXPECT_EQ(2u, tabla.size());
    } ENDM
#endif // ELKESZULT

/// Itt a vége
    if (ELKESZULT < 5 && !gtest_lite::test.fail())
      ADD_FAILURE() << "\nLegalabb ELEKSZULT == 5-ig oldja meg a feladatokat!\n";

    if (ELKESZULT >= 10 && !gtest_lite::test.fail())
      std::cout << "\nSzuper! Mind kesz" << std::endl;
    else
      std::cout << "\nEKESZULT = " << ELKESZULT << std::endl;

    GTEND(std::cerr);       // Csak C(J)PORTA működéséhez kell

    return 0;
}
