/**
 *  \file poligon.h
 *  Poligon osztály deklarációja.
 *
 *  Deklaráltuk a legfontosabb tagfüggvényeket. Néhányat meg is valósítottunk.
 *  Valósítsa meg a hiányzókat itt inline-ként, illetve a bonyolultabb
 *  függvényeket kivül definiált tagvügvényként a poligon.cpp fájlban!
 *
 *  Szükség estén bővítse az osztályt további tagfügvényekkel.
 */

#ifndef POLIGON_H
#define POLIGON_H

#include "alakzat.h"

/// Poligon osztály
/// dinamikusan tárolja a csúcspontokat
class Poligon : public Alakzat {
    Pont *pontok;       ///< csúcspontok tárolója, p0 az alaposztályban van
    size_t np;          ///< ennyi csúcsból áll
public:
    /// Konstruktor
    /// @param p0 - kezdőpont
    /// @param sz - szín és átlátszóság
    Poligon(const Pont& p0, const Szin sz);

    /// Copy konstruktor
    /// @param rhs - ezt másoljuk le
    Poligon(const Poligon& rhs);

    /// Értekadó operátor
    /// @param rhs - ezt másoljuk le
    /// @return önmaga
    Poligon& operator=(const Poligon& rhs);

    /// Csúcspontok darabszámának lekérdezése
    /// @return darab
    size_t size() const { return np; }

    /// Adott csúcspont koordinátájának lekérdezése
    /// Nem létező csp. esetén kivételt dob
    /// @param i - i. csúcspont lekérdezése
    /// @return adott csúcspont koordinátai
    Pont operator[](size_t i) const;

    /// Új csúcspont felvétele
    /// @param p - új csúcspont koordinátai
    void add(const Pont& p);

    /// Poligont rajzol
    void rajzol() const;

    ~Poligon() { delete[] pontok; }
};

/// Globális << operátor a Poligin adatainak kiírásához
/// @param os - ostream
/// @param p  - poligon
/// @return os - ostream
std::ostream& operator<<(std::ostream& os, const Poligon& p);

#endif // POLIGON_H
